<?php

namespace Drupal\views_vcards\Plugin\views\row;

use Drupal\views\Plugin\views\row\RowPluginBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Renders a vCard item based on fields.
 *
 * @ingroup views_row_plugins
 *
 * @ViewsRow(
 *   id = "views_vcard_fields",
 *   title = @Translation("vCard"),
 *   help = @Translation("Combines fields into a vCard."),
 *   theme = "views_vcards_view_row_vcard",
 *   display_types = {"vcard"}
 * )
 */
class ViewsVcardsRowPluginVcard extends RowPluginBase {

  /**
   * Does the row plugin support to add fields to it's output.
   *
   * @var bool
   */
  protected $usesFields = TRUE;

  protected function defineOptions() {
    $options = parent::defineOptions();
    
    $options['name_email']['contains'] = array(
      'first' => array('default' => ''),
      'middle' => array('default' => ''),
      'last' => array('default' => ''),
      'full' => array('default' => ''),
      'title' => array('default' => ''),
      'email' => array('default' => ''),
      'email2' => array('default' => ''),
      'email3' => array('default' => ''),
      'photo' => array('default' => ''),
    );

    $options['home']['contains'] = array(
      'home_address' => array('default' => ''),
      'home_city' => array('default' => ''),
      'home_state' => array('default' => ''),
      'home_zip' => array('default' => ''),
      'home_country' => array('default' => ''),
      'home_phone' => array('default' => ''),
      'home_cellphone' => array('default' => ''),
      'home_website' => array('default' => ''),
    );
    
    $options['work']['contains'] = array(
      'work_title' => array('default' => ''),
      'work_company' => array('default' => ''),
      'work_address' => array('default' => ''),
      'work_city' => array('default' => ''),
      'work_state' => array('default' => ''),
      'work_zip' => array('default' => ''),
      'work_country' => array('default' => ''),
      'work_phone' => array('default' => ''),
      'work_fax' => array('default' => ''),
      'work_website' => array('default' => ''),
    );
    
    return $options;
  }

  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    // Fetch field labels for the options form.
    $view_fields_labels = $this->displayHandler->getFieldLabels();

    // Do only show image fields for the photo.
    $photo_options = [];
    $fields = $this->displayHandler->getOption('fields');
    foreach ($fields as $field_name => $field) {
      if (!empty($field['type']) && $field['type'] == 'image') {
        $photo_options[$field_name] = $view_fields_labels[$field_name];
      }
    }

    $form['name_email'] = array(
      '#type' => 'details',
      '#title' => $this->t('Name and E-mail'),
      '#open' => TRUE,
    );

    $form['name_email']['first'] = array(
      '#type' => 'select',
      '#title' => $this->t('First name'),
      '#description' => $this->t('The field that is going to be used as the first name for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['name_email']['first'],
      '#required' => TRUE,
      '#empty_value' => '',
    );

    $form['name_email']['middle'] = array(
      '#type' => 'select',
      '#title' => $this->t('Middle name'),
      '#description' => $this->t('The field that is going to be used as the middle name for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['name_email']['middle'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['name_email']['last'] = array(
      '#type' => 'select',
      '#title' => $this->t('Last name'),
      '#description' => $this->t('The field that is going to be used as the last name for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['name_email']['last'],
      '#required' => TRUE,
      '#empty_value' => '',
    );

    $form['name_email']['full'] = array(
      '#type' => 'select',
      '#title' => $this->t('Full name'),
      '#description' => $this->t('The field that is going to be used as the full name for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['name_email']['full'],
      '#required' => TRUE,
      '#empty_value' => '',
    );

    $form['name_email']['title'] = array(
      '#type' => 'select',
      '#title' => $this->t('Title'),
      '#description' => $this->t('The field that is going to be used as the title for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['name_email']['title'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['name_email']['email'] = array(
      '#type' => 'select',
      '#title' => $this->t('Primary E-mail'),
      '#description' => $this->t('The field that is going to be used as the primary e-mail address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['name_email']['email'],
      '#required' => TRUE,
      '#empty_value' => '',
    );

    $form['name_email']['email2'] = array(
      '#type' => 'select',
      '#title' => $this->t('Secondary E-mail'),
      '#description' => $this->t('The field that is going to be used as the secondary e-mail address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['name_email']['email2'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['name_email']['email3'] = array(
      '#type' => 'select',
      '#description' => $this->t('The field that is going to be used as the tertiary e-mail address for each vCard.'),
      '#title' => $this->t('Tertiary E-mail'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['name_email']['email3'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['name_email']['photo'] = array(
      '#type' => 'select',
      '#description' => $this->t('The field that is going to be used as the photo for each vCard.'),
      '#title' => $this->t('Photo'),
      '#options' => $photo_options,
      '#default_value' => $this->options['name_email']['photo'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['home'] = array(
      '#type' => 'details',
      '#title' => $this->t('Home'),
      '#open' => FALSE,
    );

    $form['home']['home_address'] = array(
      '#type' => 'select',
      '#title' => $this->t('Address'),
      '#description' => $this->t('The field that is going to be used as the address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['home']['home_address'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['home']['home_city'] = array(
      '#type' => 'select',
      '#title' => $this->t('City'),
      '#description' => $this->t('The field that is going to be used as the address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['home']['home_city'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['home']['home_state'] = array(
      '#type' => 'select',
      '#title' => $this->t('State/Province'),
      '#description' => $this->t('The field that is going to be used as the address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['home']['home_state'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['home']['home_zip'] = array(
      '#type' => 'select',
      '#title' => $this->t('Zip'),
      '#description' => $this->t('The field that is going to be used as the address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['home']['home_zip'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['home']['home_country'] = array(
      '#type' => 'select',
      '#title' => $this->t('Country'),
      '#description' => $this->t('The field that is going to be used as the address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['home']['home_country'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['home']['home_phone'] = array(
      '#type' => 'select',
      '#title' => $this->t('Phone'),
      '#description' => $this->t('The field that is going to be used as the address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['home']['home_phone'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['home']['home_cellphone'] = array(
      '#type' => 'select',
      '#title' => $this->t('Cellphone'),
      '#description' => $this->t('The field that is going to be used as the address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['home']['home_cellphone'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['home']['home_website'] = array(
      '#type' => 'select',
      '#title' => $this->t('Website'),
      '#description' => $this->t('The field that is going to be used as the address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['home']['home_website'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['work'] = array(
      '#type' => 'details',
      '#title' => $this->t('Work'),
      '#open' => FALSE,
    );

    $form['work']['work_title'] = array(
      '#type' => 'select',
      '#title' => $this->t('Title'),
      '#description' => $this->t('The field that is going to be used as the address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['work']['work_title'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['work']['work_company'] = array(
      '#type' => 'select',
      '#title' => $this->t('Company'),
      '#description' => $this->t('The field that is going to be used as the address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['work']['work_company'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['work']['work_address'] = array(
      '#type' => 'select',
      '#title' => $this->t('Address'),
      '#description' => $this->t('The field that is going to be used as the address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['work']['work_address'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['work']['work_city'] = array(
      '#type' => 'select',
      '#title' => $this->t('City'),
      '#description' => $this->t('The field that is going to be used as the address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['work']['work_city'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['work']['work_state'] = array(
      '#type' => 'select',
      '#title' => $this->t('State/Province'),
      '#description' => $this->t('The field that is going to be used as the address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['work']['work_state'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['work']['work_zip'] = array(
      '#type' => 'select',
      '#title' => $this->t('Zip'),
      '#description' => $this->t('The field that is going to be used as the address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['work']['work_zip'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['work']['work_country'] = array(
      '#type' => 'select',
      '#title' => $this->t('Country'),
      '#description' => $this->t('The field that is going to be used as the address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['work']['work_country'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['work']['work_phone'] = array(
      '#type' => 'select',
      '#title' => $this->t('Phone'),
      '#description' => $this->t('The field that is going to be used as the address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['work']['work_phone'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['work']['work_fax'] = array(
      '#type' => 'select',
      '#title' => $this->t('Fax'),
      '#description' => $this->t('The field that is going to be used as the address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['work']['work_fax'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );

    $form['work']['work_website'] = array(
      '#type' => 'select',
      '#title' => $this->t('Website'),
      '#description' => $this->t('The field that is going to be used as the address for each vCard.'),
      '#options' => $view_fields_labels,
      '#default_value' => $this->options['work']['work_website'],
      '#required' => FALSE,
      '#empty_value' => '', 
    );
  }

  /**
   * {@inheritdoc}
   */
  public function validate() {
    $errors = parent::validate();
    $required_options = array(
      'name_email' => array(
        'first',
        'last',
        'full',
        'email'
      ),
    );

    foreach ($required_options as $group => $options) {
      foreach ($options as $required_option) {
        if (empty($this->options[$group][$required_option])) {
          $errors[] = $this->t('Some required fields are missing to construct a valid vCard. Check the vCard formatter settings.');
          break;
        }
      }
    }
    return $errors;
  }

  /**
   * {@inheritdoc}
   */
  public function render($row) {
    static $row_index;
    if (!isset($row_index)) {
      $row_index = 0;
    }

    // Create the vCard item object.
    $item = new \stdClass();
    $item->first_name     = $this->getField($row_index, $this->options['name_email']['first']);
    $item->middle_name    = $this->getField($row_index, $this->options['name_email']['middle']);
    $item->last_name      = $this->getField($row_index, $this->options['name_email']['last']);
    $item->full_name      = $this->getField($row_index, $this->options['name_email']['full']);
    $item->title          = $this->getField($row_index, $this->options['name_email']['title']);
    $item->email          = $this->getField($row_index, $this->options['name_email']['email']);
    $item->email2         = $this->getField($row_index, $this->options['name_email']['email2']);
    $item->email3         = $this->getField($row_index, $this->options['name_email']['email3']);
    $item->home_address   = $this->getField($row_index, $this->options['home']['home_address']);
    $item->home_city      = $this->getField($row_index, $this->options['home']['home_city']);
    $item->home_state     = $this->getField($row_index, $this->options['home']['home_state']);
    $item->home_zip       = $this->getField($row_index, $this->options['home']['home_zip']);
    $item->home_country   = $this->getField($row_index, $this->options['home']['home_country']);
    $item->home_phone     = $this->getField($row_index, $this->options['home']['home_phone']);
    $item->home_cellphone = $this->getField($row_index, $this->options['home']['home_cellphone']);
    $item->home_website   = $this->getField($row_index, $this->options['home']['home_website']);
    $item->work_title     = $this->getField($row_index, $this->options['work']['work_title']);
    $item->work_company   = $this->getField($row_index, $this->options['work']['work_company']);
    $item->work_address   = $this->getField($row_index, $this->options['work']['work_address']);
    $item->work_city      = $this->getField($row_index, $this->options['work']['work_city']);
    $item->work_state     = $this->getField($row_index, $this->options['work']['work_state']);
    $item->work_zip       = $this->getField($row_index, $this->options['work']['work_zip']);
    $item->work_country   = $this->getField($row_index, $this->options['work']['work_country']);
    $item->work_phone     = $this->getField($row_index, $this->options['work']['work_phone']);
    $item->work_fax       = $this->getField($row_index, $this->options['work']['work_fax']);
    $item->work_website   = $this->getField($row_index, $this->options['work']['work_website']);
    $item->photo          = NULL;

    $photo_field = $this->options['name_email']['photo'];
    if(!empty($photo_field)) {
      // Obtain the raw field value.
      $fid = $this->view->style_plugin->getFieldValue($row_index, $photo_field);

      if ($file = \Drupal\file\Entity\File::load($fid)) {

        // Obtain desired image style
        $image_style = $this->view->field[$photo_field]->options['settings']['image_style'];

        // Mimetype
        $type = strtoupper(str_replace('image/', '', $file->getMimeType()));

        // Use the default uri, unless there is an image style.
        $url = $file->getFileUri();
        if ($image_style) {
          $url = \Drupal\image\Entity\ImageStyle::load($image_style)->buildUrl($url);
        }

        $imgbinary = file_get_contents($url);
        $photo = "\nPHOTO;TYPE=" . $type . ';ENCODING=B:' . base64_encode($imgbinary);
        $item->photo = substr($photo, 0, 64) . "\r\n ";
        $item->photo .= chunk_split(substr($photo, 64), 63, "\r\n ");
        $item->photo = rtrim($item->photo); // Remove last newline.
      }
    }

    $row_index++;

    $build = array(
      '#theme' => $this->themeFunctions(),
      '#view' => $this->view,
      '#options' => $this->options,
      '#row' => $item,
      '#field_alias' => isset($this->field_alias) ? $this->field_alias : '',
    );

    return $build;
  }

  /**
   * Retrieves a views field value from the style plugin.
   *
   * @param $index
   *   The index count of the row as expected by views_plugin_style::getField().
   * @param $field_id
   *   The ID assigned to the required field in the display.
   *
   * @return string|null|\Drupal\Component\Render\MarkupInterface
   *   An empty string if there is no style plugin, or the field ID is empty.
   *   NULL if the field value is empty. If neither of these conditions apply,
   *   a MarkupInterface object containing the rendered field value.
   */
  public function getField($index, $field_id) {
    if (empty($this->view->style_plugin) || !is_object($this->view->style_plugin) || empty($field_id)) {
      return '';
    }
    // Convert plaintext into render array.
    $field =  $this->view->style_plugin->getField($index, $field_id);
    return is_array($field) ? $field : ['#markup' => $field];
  }

}
